-- Redstone Dust/Wire (archmage)
-- Carries power over tiles

-- we need to inherit from redstone.circuit (base class)
-- you can also inherit from subclasses, sticky pistons inherit
-- from pistons
redstone.Dust = redstone.inherit(redstone.circuit)

-- the constructor function is called after the circuit is created
function redstone.Dust:constructor()
    self.type = redstone.type.Dust
    self.img_name = "dust"
end

function redstone.Dust:update()
    -- find surrounding circuits with power and set this circuit's
    -- power to one less than the highest power
    local power = 0
    local highest_power = 0
    local c = nil
    self.on_wall_side = false
    
    -- check all adjacent non-diagonal tiles
    for i = 0, 3 do
        c = redstone.get_circuit(self.x + redstone.rot[i][1], self.y + redstone.rot[i][2])
        -- check if there is a circuit
        if ( c ~= nil ) then
            -- get the circuits power according to our displacement
            -- for example repeaters should only give power in one direction
            power = c:get("power", self, i)
            if ( power > highest_power ) then
                highest_power = power
            end
        end
    end
    
    -- subtract one from the highest power and be sure it's not negative
    highest_power = ((highest_power - 1) < 0 and 0 or (highest_power-1))
    local on = self.power > 0
    local turn_on = highest_power > 0
    
    self.power = highest_power
    
    -- if the power differs from our current power then update
    if ( on and not turn_on ) then
        self:update_image()
    end
    
    if ( (not on) and turn_on ) then
        self:update_image()
    end
end
