-- Redstone Piston (archmage)
-- Can push (but not pull) dynamic objects

redstone.Piston = redstone.inherit(redstone.circuit)
function redstone.Piston:constructor()
    self.type = redstone.type.Piston
    self.img_name = "piston"
    self.img2_name = "piston_push.bmp"
    self.img2id = 0
end

function redstone.Piston:get_full_image_name()
    return self.img_name.."_"..((self.power > 0) and "1" or "0")
end

function redstone.Piston:calc_power()
    -- pistons do not ever have power
    return 0
end

function redstone.Piston:deconstructor()
    -- make sure to remove the piston image
    if ( self.img2id ~= 0 ) then
        freeimage(self.img2id)
        self.img2id = 0
    end
end

function redstone.Piston:power_on()
    local can_push = true
    local obj = 0
    local x, y = self.x + redstone.rot[self.rot][1], self.y + redstone.rot[self.rot][2]
    local x2, y2 = x+redstone.rot[self.rot][1], y+redstone.rot[self.rot][2]
    
    -- check for circuits in the way
    local c = redstone.get_circuit(x, y)
    if ( c ~= nil ) then
        -- remove it
        c:remove()
    end
    
    -- check for an object to push
    for _, o in pairs(object(0, "table")) do
        local ox, oy = object(o, "tilex"), object(o, "tiley")
        if ( ox == x and oy == y ) then
            obj = o
            break;
        end
    end
    
    -- is there something in the way?
    if ( not tile(x, y, "walkable") ) then
        -- is it an object or a wall/obstacle?
        if ( obj == 0 ) then
            can_push = false
        end
    end
    
    if ( not tile(x2, y2, "walkable") ) then
        if ( obj ~= 0 ) then
            can_push = false
        end
    end
    
    if ( can_push ) then
        self.img2id = image("gfx/redstone/"..self.img2_name, x*32+16, y*32+16, 0)
        imagepos(self.img2id, x*32+16, y*32+16, self.rot*90)
        self:update_image();
        if ( obj ~= 0 ) then
            parse("spawnobject "..object(obj, "type").." "..x2.." "..y2.." "..object(obj, "rot").." "..object(obj, "mode").." "..object(obj, "team").." "..object(obj, "player"))
            parse("killobject "..obj)
        end
    end
end

function redstone.Piston:power_off()
    self:update_image()
    if ( self.img2id ~= 0 ) then
        freeimage(self.img2id)
        self.img2id = 0
    end
end

function redstone.Piston:update()
    local power = 0
    local highest_power = 0
    local c = nil
    self.on_wall_side = false
    
    -- check all adjacent non-diagonal tiles
    for i = 0, 3 do
        c = redstone.get_circuit(self.x + redstone.rot[i][1], self.y + redstone.rot[i][2])
        -- check if there is a circuit
        if ( c ~= nil ) then
            -- get the circuits power according to our displacement
            -- for example repeaters should only give power in one direction
            power = c:get("power", self, i)
            if ( power > highest_power ) then
                highest_power = power
            end
        end
    end
    
    local is_on = self.power > 0
    local power_on = highest_power > 0
    if ( is_on and not power_on ) then
        -- power off
        self.power = 0
        self:power_off()
    end
    
    if ( (not is_on) and power_on ) then
        -- power on
        self.power = 1
        self:power_on()
    end
end
