-- Redstone Torch (archmage)
-- Acts as a power source

redstone.Torch = redstone.inherit(redstone.circuit)
function redstone.Torch:constructor()
    self.type = redstone.type.Torch
    self.img_name = "torch"
    self.power = redstone.torch_power
end

function redstone.Torch:get_full_image_name()
    return self.img_name.."_"..((self.power>0) and "1" or "0").."_"..(self.on_wall_side and "1" or "0")
end

function redstone.Torch:calc_power(to, d)
    -- torches don't transfer power down
    if ( self.on_wall_top ) then
        if ( to.on_wall_top ) then
            return self.power
        else
            return 0
        end
    elseif ( self.on_wall_side ) then
        if ( (not to.on_wall_top) and (not to.on_wall_side) ) then
            return self.power
        else
            return 0
        end
    end
    return self.power
end

function redstone.Torch:update()
    local turn_off = false
    if ( self.on_wall_top ) then
        for i = 0, 3 do
            local c = redstone.get_circuit(self.x+redstone.rot[i][1], self.y+redstone.rot[i][2])
            if ( c ~= nil ) then
                if ( not c.on_wall_top ) then
                    if ( c:get("power", self, i) > 0 ) then
                        turn_off = true
                    end
                end
            end
        end
    elseif ( self.on_wall_side ) then
        -- position of the wall that the torch is attached to
        local wx, wy = self.x + redstone.rot[self.rot][1], self.y + redstone.rot[self.rot][2]
        local c = redstone.get_circuit(wx, wy) 
        if ( c ~= nil ) then
            if ( c:get("power", self, self.rot) > 0 ) then
                turn_off = true
            end
        end
    end
    
    local last = self.power
    
    if ( turn_off ) then
        self.power = 0
        if ( last > 0 ) then
            self:update_image()
        end
    else
        self.power = redstone.torch_power
        if ( last == 0 ) then
            self:update_image()
        end
    end
end
