-- MineCraft Redstone Mod
-- By archmage (unrealsoftware.de/profile.php?userid=13445)
redstone = {}

-- [[ CONFIG ]] --
redstone.updates_per_sec = 30
redstone.torch_power = 16
------------------

redstone.clock = 0
redstone.rot = {[0]={0, -1}, {1, 0}, {0, 1}, {-1, 0}}
redstone.get_angle =    function(t)
                            for i = 0, 3 do
                                if ( redstone.rot[i][1] == t[1] and redstone.rot[i][2] == t[2] ) then
                                    return i
                                end
                            end
                            return -1
                        end
redstone.opp_angle =    function (x)
                            return (x+2) % 4
                        end

redstone.circuit_map = {}           -- used to quickly find circuits on the map
redstone.circuits = {}
redstone.count = 0
redstone.type = {NULL = 0}
redstone.name = {[0]="NULL"}
redstone.price = {[0]=0}
redstone.class = {[0]=nil}
redstone.circuit = {x = 0, y = 0, type=0, rot=0, power=0, id = 0, imgid = 0, on_wall_top=false, on_wall_side=false, img_name="redstone"}
redstone.circuit.mt = {__index = redstone.circuit}

function redstone.add_circuit(class, name, price)
    local index = redstone.count+1
    redstone.type[name] = index
    redstone.name[index] = name
    redstone.price[index] = price
    redstone.class[index] = class
    redstone.count = redstone.count + 1
end

redstone.menu =         function (p)
                            local str = "Redstone Circuit Buy Menu ("..p+1 ..")"
                            if ( redstone.count <= 9 ) then
                                for i = 1, redstone.count do
                                    str = str..","..redstone.name[i].."|$"..redstone.price[i]
                                end
                            else
                                local start = p*6+1
                                for i = start, start+6 do
                                    if ( i <= redstone.count ) then
                                        str = str..","..redstone.name[i].."|$"..redstone.price[i]
                                    else
                                        str = str..","
                                    end
                                end
                                str = str..",Next Page,Previous Page"
                            end
                            return str
                        end

function redstone.get_circuit(x, y)
    if ( redstone.circuit_map[x] == nil ) then
        return nil
    elseif ( redstone.circuit_map[x][y] == nil ) then
        return nil
    else
        return redstone.circuits[redstone.circuit_map[x][y]]
    end
end

function redstone.circuit:new(x, y)
    local inst = {}
    setmetatable(inst, {__index = self})
    inst.id = #redstone.circuits+1; inst.type = 0; inst.x = x; inst.y = y; inst.rot = rot

    if ( redstone.circuit_map[x] == nil ) then
        redstone.circuit_map[x] = {}
    end
    
    if ( redstone.circuit_map[x][y] == nil ) then
        redstone.circuit_map[x][y] = inst.id
    elseif ( redstone.circuits[redstone.circuit.map[x][y]] == nil ) then
        redstone.circuit_map[x][y] = inst.id
    else
        return nil
    end
    
    redstone.circuits[#redstone.circuits+1] = inst
    inst:constructor()
    inst:update_image()
    return inst
end

function redstone.circuit:remove()
    if ( self == nil ) then return nil end
    local x, y = self.x, self.y
    
    self:deconstructor()
    
    if ( redstone.circuit_map[x] ~= nil ) then
        if ( redstone.circuit_map[x][y] ~= nil ) then
            redstone.circuit_map[x][y] = nil
        end
    end
    
    self:free_image()
    redstone.circuits[self.id] = nil
end

function redstone.circuit:constructor()
    -- after being created and intialized
    -- should set self.type to the appropriate value if nothing else
end

function redstone.circuit:deconstructor()
    -- before being destroyed
end

function redstone.circuit:get_full_image_name()
    return self.img_name.."_"..((self.power > 0) and "1" or "0")
end

function redstone.circuit:get_img_pos()
    return self.x*32+16, self.y*32+16
end

function redstone.circuit:update_image()
    -- free the circuit's image if it exists
    if ( self.imgid ~= 0 ) then
        freeimage(self.imgid)
        self.imgid = 0
    end
    -- get image position with offset
    local x, y = self:get_img_pos()
    -- load image
    self.imgid = image("gfx/redstone/"..(self:get_full_image_name())..".bmp", x, y, self.on_wall_top and 3 or 0)
    -- set rotation
    imagepos(self.imgid, x, y, self.rot * 90)
end

function redstone.circuit:free_image()
    if ( self.imgid ~= 0 ) then
        freeimage(self.imgid)
        self.imgid = 0
    end
end

function redstone.circuit:calc_power(to, d)
    -- to is a circuit
    -- d should be either 0,1,2,3
    -- can get displacement with redstone.rot[redstone.opp_angle(d)]
    -- some circuits may give different power base of displacement (d)
    return self.power
end

-- get a value from a circuit
-- in the case of power 'to' is the circuit that's trying to recieve power
-- and 'displacement' is the positional offset of the two circuits
function redstone.circuit:get(what, to, displacement)
    if ( what == "power" ) then
        if ( to == nil ) then
            return self.power
        else
            -- calculate power to give
            return self:calc_power(to, displacement)
        end
    else
        return self[what]
    end
end

-- when the circuit recieves power (called once while it has power)
-- not called when the circuit is first placed
-- you have to call this in update()!
function redstone.circuit:power_on()
end

-- when the circuit loses power (called once while it doesn't have power)
-- not called when the circuit is first placed
-- you have to call this in update()!
function redstone.circuit:power_off()
end

function redstone.circuit:set(what, to)
    if ( what == "power" ) then
        if ( to < 0 ) then
            to = 0
        end
    end
    self[what] = to
end
 
function redstone.circuit:use()
    -- when the circuit is used by a player ('E' key)
end

function redstone.circuit:update()
    -- when being updated
end

dofile("sys/lua/rs/circuits.lua")
dofile("sys/lua/rs/hooks.lua")